import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.*;

/**
 * Celestial Simulator
 * 
 * @author Kevin Minter
 * @version 5/25/12
 */
public class Body extends SmoothMover
{
    // constants
    private static final double GRAVITY = 5.8;

    // fields
    private double mass;
    private boolean sun=false;
    public boolean transparent=false;

    GreenfootImage base;

    /**
     * Constructs a body smaller than a sun in orbit around nearest bigger object
     */
    public Body(int size, double speed, double mass, Color color,List<Body> bodies, int x,int y)
    {
        int delta=10000;
        Body closest = null;
        int xDif,yDif;
        for(Body body : bodies)
        {
            if(body != this)
            {
                if(body.getMass()>=mass)
                {
                    if(body.getX()>x)
                    {
                        xDif = body.getX() - x;
                    }
                    else
                    {
                        xDif = x - body.getX();
                    }

                    if(body.getY()>y)
                    {
                        yDif =  body.getY() - y;
                    }
                    else
                    {
                        yDif = y - body.getY();
                    }

                    if(xDif+yDif<delta)
                    {
                        delta = xDif+yDif;
                        closest = body;
                    }
                }
            }
        }
        int cX = closest.getX();
        int cY = closest.getY();
        int rotation = getRotation();
        turnTowards(cX,cY);
        int cRotation = getRotation();
        setRotation(rotation);
        Vector movement;
        if(x>=cX)
        {
            if(cY>=y)
            {
                movement = new Vector(cRotation-rotation,speed);
            }
            else
            {
                movement = new Vector(cRotation+rotation-60,speed);
            }
        }
        else
        {
            if(cY>=y)
            {
                movement = new Vector(cRotation+rotation+60,speed);
            }
            else
            {
                movement = new Vector(cRotation-rotation/2,speed);
            }
        }
        this.mass = mass;
        addForce(movement);
        GreenfootImage image = new GreenfootImage (size, size);
        image.setColor (color);
        image.fillOval (0, 0, size-1, size-1);
        base = image;
        setImage (image);
    }

    /**
     * Construct a Body with a specified size, mass, movement and color.
     */
    public Body(int size, double mass, Vector movement, Color color, boolean sun)
    {
        this.mass = mass;
        this.sun = sun;
        addForce(movement);
        GreenfootImage image = new GreenfootImage (size, size);
        image.setColor (color);
        image.fillOval (0, 0, size-1, size-1);
        base = image;
        setImage (image);
    }

    /**
     * Act. That is: apply  the gravitation forces from
     * all other bodies around, and then move.
     */
    public void act() 
    {
        Space world = (Space) getWorld();
        if(world.getPlay()=="Play")
        {
            if(transparent==false)
            {
                applyForces();
            }
            if(sun==false)
            {
                move();
                checkEdges();
                checkCollisions();
            }
        }
    }

    /**
     * Return the mass of this body.
     */
    public double getMass()
    {
        return mass;
    }

    /**
     * Applies gravity from all the other bodies
     */
    private void applyForces()
    {
        List<Body> bodies = getWorld().getObjects(Body.class);

        for(Body body : bodies)
        {
            if(body != this)
            {
                applyGravity(body);    
            }
        }
    }

    /**
     * Applies gravity from one other body to the current body
     */
    private void applyGravity(Body other)
    {
        double dx = other.getExactX() - this.getExactX();
        double dy = other.getExactY() - this.getExactY();
        Vector force = new Vector(dx,dy);
        double distance = Math.sqrt (dx*dx+dy*dy);
        double strength = GRAVITY * this.mass * other.mass / (distance* distance);
        double acceleration = strength / this.mass;
        force.setLength(acceleration);
        addForce(force);

    }

    /**
     * Checks for collisions with bodies with mass equal to or greater
     */
    private void checkCollisions()
    {
        Actor actor=this.getOneIntersectingObject(Actor.class);

        if(actor!=null&&actor instanceof Body)
        {
            Body body = (Body)actor;
            if( ( body.getMass()>this.getMass() ) && ( body.getTransparent()==false) )
            {
                getWorld().removeObject(this);
            }

            if( ( body.getMass()==this.getMass() ) && ( body.getTransparent()==false) )
            {
                getWorld().removeObject(body);
                getWorld().removeObject(this);
            }
        }
    }

    /**
     * Checks for bodies reaching the edge to hide it and stop applying gravity
     */
    public void checkEdges()
    {
        Space w = (Space) getWorld();
        if(transparent==false)
        {
            if( getX()<1 || getY()<1 || getX()>w.getWidth()-2 || getY()>w.getHeight()-2 )
            { 
                transparent=true;
                setImage("transparent.png");
            }
        }

        if(transparent==true)
        {
            if(getX()>1 && getX()<w.getWidth()-2 && getY()>1 && getY()<w.getHeight()-2 )
            {
                transparent=false;
                setImage(base);
            }
        }
    }

    /**
     * Returns if the body is off screen
     */
    public boolean getTransparent()
    {
        return transparent;
    }
}
